use super::{body::Body, header::Header, Command, MessageType};
use std::ops::Deref;

pub trait RequestSerializer {
    fn message_type(&self) -> MessageType;
    fn body_type(&self) -> u8;

    fn serialize(&self) -> Vec<u8>;
}

#[derive(Clone)]
pub struct CommandRequest {
    #[cfg(debug_assertions)]
    pub name: &'static str,

    command: Command,
}

impl CommandRequest {
    fn checksum(data: &[u8]) -> u8 {
        ((!data.iter().map(|&b| b as u16).sum::<u16>() + 1) & 0xFF) as u8
    }

    pub fn new(
        device_protocol_version: u8,
        device_type: u8,
        message_type: MessageType,
        body: Body,
        #[cfg(debug_assertions)] name: &'static str,
    ) -> Self {
        Self {
            command: Command {
                header: Header::new(
                    body.len() as u8,
                    device_type,
                    device_protocol_version,
                    message_type,
                ),
                body,
            },

            #[cfg(debug_assertions)]
            name,
        }
    }

    #[allow(unused)]
    pub fn header(&self) -> &Header {
        &self.command.header
    }

    #[allow(unused)]
    pub fn body(&self) -> &Body {
        &self.command.body
    }

    pub fn body_mut(&mut self) -> &mut Body {
        &mut self.command.body
    }
}

impl RequestSerializer for CommandRequest {
    fn serialize(&self) -> Vec<u8> {
        let mut stream = Vec::new();

        stream.extend_from_slice(self.command.header.raw());
        stream.extend_from_slice(self.command.body.raw());
        stream.push(Self::checksum(&stream[1..]));

        stream
    }

    fn message_type(&self) -> MessageType {
        self.header().message_type()
    }

    fn body_type(&self) -> u8 {
        self.body().body_type()
    }
}

pub struct CommandQuerySubtype {
    command: CommandRequest,
}

impl CommandQuerySubtype {
    pub fn new(device_type: u8) -> Self {
        Self {
            command: CommandRequest::new(
                0,
                device_type,
                MessageType::QuerySubtype,
                Body::from(([0x00; 18].as_slice(), 0x00)),
                #[cfg(debug_assertions)]
                "QuerySubtype",
            ),
        }
    }

    pub fn request(self) -> CommandRequest {
        self.command
    }
}

impl Deref for CommandQuerySubtype {
    type Target = CommandRequest;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}

impl RequestSerializer for CommandQuerySubtype {
    fn serialize(&self) -> Vec<u8> {
        self.command.serialize()
    }

    fn message_type(&self) -> MessageType {
        self.header().message_type()
    }

    fn body_type(&self) -> u8 {
        self.body().body_type()
    }
}

pub struct CommandQueryCustom {
    command: CommandRequest,
}

impl CommandQueryCustom {
    pub fn new(device_type: u8, message_type: MessageType, body: Body) -> Self {
        Self {
            command: CommandRequest::new(
                0,
                device_type,
                message_type,
                body,
                #[cfg(debug_assertions)]
                "QueryCustom",
            ),
        }
    }

    #[allow(unused)]
    pub fn request(self) -> CommandRequest {
        self.command
    }
}

impl Deref for CommandQueryCustom {
    type Target = CommandRequest;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}

impl RequestSerializer for CommandQueryCustom {
    fn serialize(&self) -> Vec<u8> {
        self.command.serialize()
    }

    fn message_type(&self) -> MessageType {
        self.header().message_type()
    }

    fn body_type(&self) -> u8 {
        self.body().body_type()
    }
}

pub struct CommandHeartbeat;

impl RequestSerializer for CommandHeartbeat {
    fn serialize(&self) -> Vec<u8> {
        vec![0x00]
    }

    fn message_type(&self) -> MessageType {
        MessageType::None
    }

    fn body_type(&self) -> u8 {
        0
    }
}
