use std::{
    collections::HashMap,
    ops::{Deref, DerefMut},
};

use anyhow::Result;

use crate::command::*;

use super::{AttributeValue, DeviceBackend};

pub struct E1 {
    modes: HashMap<u8, String>,
    attributes: HashMap<&'static str, AttributeValue>,

    status: [&'static str; 5],
    progress: [&'static str; 6],

    device_protocol_version: u8,
}

impl E1 {
    pub fn new() -> Result<Self> {
        let modes = [
            (0x0, "Neutral Gear"),
            (0x1, "Auto"),
            (0x2, "Heavy"),
            (0x3, "Normal"),
            (0x4, "Energy Saving"),
            (0x5, "Delicate"),
            (0x6, "Hour"),
            (0x7, "Quick"),
            (0x8, "Rinse"),
            (0x9, "90min"),
            (0xA, "Self Clean"),
            (0xB, "Fruit Wash"),
            (0xC, "Self Define"),
            (0xD, "Germ"),
            (0xE, "Bowl Wash"),
            (0xF, "Kill Germ"),
            (0x10, "Sea Food Wash"),
            (0x12, "Hot Pot Wash"),
            (0x13, "Quiet"),
            (0x14, "Less Wash"),
            (0x16, "Oil Net Wash"),
            (0x19, "Cloud Wash"),
        ]
        .into_iter()
        .map(|(i, s)| (i, s.to_string()))
        .collect();

        let attributes = [
            ("power", AttributeValue::Bool(false)),
            ("status", AttributeValue::String(None)),
            ("mode", AttributeValue::String(None)),
            ("additional", AttributeValue::Int(0)),
            ("uv", AttributeValue::Bool(false)),
            ("dry", AttributeValue::Bool(false)),
            ("dry_status", AttributeValue::Bool(false)),
            ("door_closed", AttributeValue::Bool(false)),
            ("rinse_aid_shortage", AttributeValue::Bool(false)),
            ("salt_shortage", AttributeValue::Bool(false)),
            ("child_lock", AttributeValue::Bool(false)),
            ("storage", AttributeValue::Bool(false)),
            ("storage_progress", AttributeValue::Bool(false)),
            ("time_remaining", AttributeValue::String(None)),
            ("progress", AttributeValue::String(None)),
            ("storage_remaining", AttributeValue::String(None)),
            ("temperature", AttributeValue::String(None)),
            ("humidity", AttributeValue::String(None)),
            ("waterswitch", AttributeValue::Bool(false)),
            ("water_lack", AttributeValue::Bool(false)),
            ("error_code", AttributeValue::String(None)),
            ("softwater", AttributeValue::Int(0)),
            ("wrong_operation", AttributeValue::String(None)),
            ("bright", AttributeValue::Int(0)),
        ]
        .into_iter()
        .collect();

        let status = ["Off", "Idle", "Delay", "Running", "Error"];
        let progress = ["Idle", "Pre-wash", "Wash", "Rinse", "Dry", "Complete"];

        Ok(Self {
            modes,
            attributes,
            status,
            progress,

            device_protocol_version: 0,
        })
    }
}

impl DeviceBackend for E1 {
    fn build_query(&self) -> CommandRequest {
        CommandQuery::new(self.device_protocol_version).request()
    }

    fn process_message(&mut self, msg: &[u8]) {
        CommandE1Response::new(msg).update_attributes(
            &mut self.attributes,
            &self.modes,
            &self.status,
            &self.progress,
        );
    }

    fn set_attribute(&self, attribute: &str, value: AttributeValue) -> CommandRequest {
        match attribute {
            "power" => {
                let mut cmd = CommandPower::new(self.device_protocol_version);
                cmd.set_power(value.bool());

                cmd.request()
            }
            "child_lock" => {
                let mut cmd = CommandLock::new(self.device_protocol_version);
                cmd.set_lock(value.bool());

                cmd.request()
            }
            "storage" => {
                let mut cmd = CommandStorage::new(self.device_protocol_version);
                cmd.set_storage(value.bool());

                cmd.request()
            }

            _ => todo!("attribute: {attribute}"),
        }
    }

    fn attributes(&self) -> &HashMap<&'static str, AttributeValue> {
        &self.attributes
    }
}

pub struct CommandE1Base {
    command: CommandRequest,
}

impl CommandE1Base {
    pub fn new(
        device_protocol_version: u8,
        message_type: MessageType,
        body: Body,
        #[cfg(debug_assertions)] name: &'static str,
    ) -> Self {
        Self {
            command: CommandRequest::new(
                device_protocol_version,
                0xE1,
                message_type,
                body,
                #[cfg(debug_assertions)]
                name,
            ),
        }
    }
}

impl Deref for CommandE1Base {
    type Target = CommandRequest;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}

impl DerefMut for CommandE1Base {
    fn deref_mut(&mut self) -> &mut Self::Target {
        &mut self.command
    }
}

pub struct CommandPower {
    command: CommandE1Base,
}

impl CommandPower {
    pub fn new(device_protocol_version: u8) -> Self {
        Self {
            command: CommandE1Base::new(
                device_protocol_version,
                MessageType::Set,
                Body::from(([0x00; 4].as_slice(), 0x08)),
                #[cfg(debug_assertions)]
                "Power",
            ),
        }
    }

    pub fn set_power(&mut self, power: bool) {
        self.command.body_mut()[1] = if power { 0x01 } else { 0x00 };
    }

    pub fn request(self) -> CommandRequest {
        self.command.command
    }
}

impl Deref for CommandPower {
    type Target = CommandRequest;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}

pub struct CommandLock {
    command: CommandE1Base,
}

impl CommandLock {
    pub fn new(device_protocol_version: u8) -> Self {
        Self {
            command: CommandE1Base::new(
                device_protocol_version,
                MessageType::Set,
                Body::from(([vec![0x04], vec![0x00; 36]].concat().as_slice(), 0x08)),
                #[cfg(debug_assertions)]
                "Lock",
            ),
        }
    }

    pub fn set_lock(&mut self, lock: bool) {
        self.command.body_mut()[1] = if lock { 0x03 } else { 0x04 }
    }

    pub fn request(self) -> CommandRequest {
        self.command.command
    }
}

impl Deref for CommandLock {
    type Target = CommandRequest;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}

pub struct CommandStorage {
    command: CommandE1Base,
}

impl CommandStorage {
    pub fn new(device_protocol_version: u8) -> Self {
        Self {
            command: CommandE1Base::new(
                device_protocol_version,
                MessageType::Set,
                Body::from((
                    [vec![0x00; 4], vec![0xFF; 6], vec![0x00; 27]]
                        .concat()
                        .as_slice(),
                    0x81,
                )),
                #[cfg(debug_assertions)]
                "Storage",
            ),
        }
    }

    pub fn set_storage(&mut self, storage: bool) {
        self.command.body_mut()[4] = if storage { 0x01 } else { 0x00 }
    }

    pub fn request(self) -> CommandRequest {
        self.command.command
    }
}

impl Deref for CommandStorage {
    type Target = CommandRequest;

    fn deref(&self) -> &Self::Target {
        &self.command
    }
}

pub struct CommandQuery {
    command: CommandE1Base,
}

impl CommandQuery {
    pub fn new(device_protocol_version: u8) -> Self {
        Self {
            command: CommandE1Base::new(
                device_protocol_version,
                MessageType::Query,
                Body::from(([].as_slice(), 0x00)),
                #[cfg(debug_assertions)]
                "Query",
            ),
        }
    }

    pub fn request(self) -> CommandRequest {
        self.command.command
    }
}

pub struct CommandE1Response {
    command: CommandResponse,
    start: bool,
}

impl CommandE1Response {
    pub fn new(msg: &[u8]) -> Self {
        let command = CommandResponse::new(msg);

        Self {
            command,
            start: false,
        }
    }

    pub fn update_attributes(
        &mut self,
        attributes: &mut HashMap<&'static str, AttributeValue>,
        modes: &HashMap<u8, String>,
        status: &[&str],
        progress: &[&str],
    ) {
        let message_type = self.command.header().message_type();
        let body = self.command.body();
        let body_type = self.command.body().body_type();

        if (message_type == MessageType::Set && body_type <= 7)
            || ((message_type == MessageType::Query || message_type == MessageType::Notify1)
                && body_type == 0)
        {
            attributes.get_mut("power").unwrap().set(
                #[cfg(debug_assertions)]
                "power",
                body[1] > 0,
            );
            attributes.get_mut("status").unwrap().set(
                #[cfg(debug_assertions)]
                "status",
                status[body[1] as usize],
            );
            attributes.get_mut("mode").unwrap().set(
                #[cfg(debug_assertions)]
                "mode",
                &modes[&body[2]],
            );
            attributes.get_mut("additional").unwrap().set(
                #[cfg(debug_assertions)]
                "additional",
                body[3],
            );

            // 0 - open, 1 - close
            attributes.get_mut("door_closed").unwrap().set(
                #[cfg(debug_assertions)]
                "door_closed",
                (body[5] & 0x01) == 0,
            );

            // 0 - enough, 1 - shortage
            attributes.get_mut("rinse_aid_shortage").unwrap().set(
                #[cfg(debug_assertions)]
                "rinse_aid_shortage",
                (body[5] & 0x02) > 0,
            );

            // 0 - enough, 1 - shortage
            attributes.get_mut("salt_shortage").unwrap().set(
                #[cfg(debug_assertions)]
                "salt_shortage",
                (body[5] & 0x04) > 0,
            );

            let start_pause = (body[5] & 0x08) > 0;

            if start_pause {
                self.start = true;
            } else if attributes["status"].str() == Some("Delay")
                || attributes["status"].str() == Some("Running")
            {
                self.start = false;
            }

            attributes.get_mut("child_lock").unwrap().set(
                #[cfg(debug_assertions)]
                "child_lock",
                (body[5] & 0x10) > 0,
            );
            attributes.get_mut("uv").unwrap().set(
                #[cfg(debug_assertions)]
                "uv",
                (body[4] & 0x02) > 0,
            );
            attributes.get_mut("dry").unwrap().set(
                #[cfg(debug_assertions)]
                "dry",
                (body[4] & 0x10) > 0,
            );
            attributes.get_mut("dry_status").unwrap().set(
                #[cfg(debug_assertions)]
                "dry_status",
                (body[4] & 0x20) > 0,
            );
            attributes.get_mut("storage").unwrap().set(
                #[cfg(debug_assertions)]
                "storage",
                (body[5] & 0x20) > 0,
            );
            attributes.get_mut("storage_progress").unwrap().set(
                #[cfg(debug_assertions)]
                "storage_progress",
                (body[5] & 0x40) > 0,
            );
            attributes.get_mut("time_remaining").unwrap().set(
                #[cfg(debug_assertions)]
                "time_remaining",
                body[6],
            );
            attributes.get_mut("progress").unwrap().set(
                #[cfg(debug_assertions)]
                "progress",
                progress[body[9] as usize],
            );
            attributes.get_mut("storage_remaining").unwrap().set(
                #[cfg(debug_assertions)]
                "storage_remaining",
                if body.len() > 18 {
                    body[11]
                } else {
                    false.into()
                },
            );
            attributes.get_mut("temperature").unwrap().set(
                #[cfg(debug_assertions)]
                "temperature",
                body[11],
            );
            attributes.get_mut("humidity").unwrap().set(
                #[cfg(debug_assertions)]
                "humidity",
                if body.len() > 33 {
                    body[33].into()
                } else {
                    AttributeValue::from(None)
                },
            );
            attributes.get_mut("waterswitch").unwrap().set(
                #[cfg(debug_assertions)]
                "waterswitch",
                (body[4] & 0x04) > 0,
            );
            attributes.get_mut("water_lack").unwrap().set(
                #[cfg(debug_assertions)]
                "water_lack",
                (body[5] & 0x80) > 0,
            );
            attributes.get_mut("error_code").unwrap().set(
                #[cfg(debug_assertions)]
                "error_code",
                body[10],
            );
            attributes.get_mut("softwater").unwrap().set(
                #[cfg(debug_assertions)]
                "softwater",
                body[13],
            );
            attributes.get_mut("wrong_operation").unwrap().set(
                #[cfg(debug_assertions)]
                "wrong_operation",
                body[16],
            );
            attributes.get_mut("bright").unwrap().set(
                #[cfg(debug_assertions)]
                "bright",
                if body.len() > 24 {
                    body[24].into()
                } else {
                    AttributeValue::from(None)
                },
            );
        }
    }
}
